;********************************************************************************************
;
;	ROUTINES
;
;********************************************************************************************

;********************************************************************************************
;
;	- SETPORTS
;	- VAR_INIT
;	- DECODE
;
;********************************************************************************************

;****************************************************************************
;****************************************************************************
;****************************************************************************

SETPORTS
	
	CLRF	INTCON		;;
        movlw   B'00000110'	;;
        movwf   ADCON1  	;; digital I/O PORTA

        movlw   B'11111111'	;;
        movwf   TRISA   	;; PORTA
        movlw   B'11111111'	;;
        movwf   TRISB   	;; PORTB
        movlw   B'11111111'	;;
        movwf   TRISC   	;; PORTC

        BSF	INTCON2, 7	;; DISABLE pull-ups 
	
	BCF	TRIS_LED, LED_RED
	BCF	TRIS_LED, LED_GREEN

	BCF	R_LED		;;
	BCF	G_LED		;;

	CALL	SERVO_INIT	;; SETTING OUT SERVI
	
        CLRF	PIE1            ;; interrupt periferici
        CLRF	PIE2            ;;

	DEV_SETTING		;;
	CALL INIT_KEYS		;;
	RETURN			;;


;****************************************************************************
VAR_INIT
	CLRF	CINQUANTESIMI	;
	CLRF	QUARTI		;
	CLRF	SECONDI		;
	CLRF	MINUTI		;

	MOVLW	START_SPEED	;<----
	MOVWF	XSPEED		;<----
	MOVWF	XNUM_SPEED	;<----
	CLRF	MOV		;
	CLRF	STATUS_SEQ_NUM	;
	CLRF	EMU_MODE	;

RES_VAL	
	MOVLW	0x80		; SERVI CENTRATI
	MOVWF	VALUE_SERV_0	;
	MOVWF	VALUE_SERV_1	;
	MOVWF	VALUE_SERV_2	;
	MOVWF	VALUE_SERV_3	;
	MOVWF	VALUE_SERV_4	;
	MOVWF	VALUE_SERV_5	;
	MOVWF	VALUE_SERV_6	;
	MOVWF	VALUE_SERV_7	;

	CLRF	STEP_COUNTER_L			;
	CLRF	STEP_COUNTER_H			;

;	USARE STATUS_SEQ_NUM PER INIZIALIZZARE!!!! QUI!!!
;	MISURARE IL VALORE DI STATUS_SEQ_NUM!!!

	CALL	LOAD_TOC			;
	CALL	STATUS_SEQ_NUM_READ		;
	CALL	SET_SEQ_START_PARAM		;

	RETURN

;****************************************************************************
;****************************************************************************

STATUS_SEQ_NUM_READ:
	CLRF	STATUS_SEQ_NUM_TEMP	;
	BTFSS	SEQ_IN_0		;
	BSF	STATUS_SEQ_NUM_TEMP, 0	;
	BTFSS	SEQ_IN_1		;
	BSF	STATUS_SEQ_NUM_TEMP, 1	;
	BTFSS	SEQ_IN_2		;
	BSF	STATUS_SEQ_NUM_TEMP, 2	;
	BTFSS	SEQ_IN_3		;
	BSF	STATUS_SEQ_NUM_TEMP, 3	;
	MOVF	STATUS_SEQ_NUM_TEMP, W	;
	MOVWF	STATUS_SEQ_NUM		;

	MOVF	EMU_STATUS_SEQ_NUM, W	; <--- EMULATOR SEQUENCE STATUS NUMBER! 
	BTFSC	EMU_MODE, 0		; IF EMULATOR MODE = 1 THEN
	MOVWF	STATUS_SEQ_NUM		; SEQUENCE STATUS NUMBER = EMULATOR SEQUENCE STATUS NUMBER 
	RETURN



;****************************************************************************
;****************************************************************************
;****************************************************************************




;***************************************************************
;
;	 CODIFICA COMUNICAZIONE 
;
;***************************************************************

_STOP	=	B'00000000'		;
_START	=	B'00000001'		;
_LOAD = 	B'00000010'		; UPDATE MEMORY
_START_MOV =	B'00000011'		;
_STOP_MOV =	B'00000100'		;
_SPEED_MOV =	B'00000101'		; UPDATE SPEED
_DOWNLOAD =	B'00000110'		; DOWNLOAD MEMORY
_ERASE =	B'00000111'		; ERASE MEMORY
_PC_EMU =	B'00010000'		; PC EMULATOR
_RESET =	B'00010001'		; RESET PROCESSOR

_S_0	=	B'00001000'		; UPDATE SERVO 0
_S_1	=	B'00001001'		; UPDATE SERVO 1
_S_2	=	B'00001010'		; UPDATE SERVO 2
_S_3	=	B'00001011'		; UPDATE SERVO 3
_S_4	=	B'00001100'		; UPDATE SERVO 4
_S_5	=	B'00001101'		; UPDATE SERVO 5
_S_6	=	B'00001110'		; UPDATE SERVO 6
_S_7	=	B'00001111'		; UPDATE SERVO 7

_S_8	=	B'00101000'		; UPDATE SERVO 8
_S_9	=	B'00101001'		; UPDATE SERVO 9
_S_10	=	B'00101010'		; UPDATE SERVO 10
_S_11	=	B'00101011'		; UPDATE SERVO 11
;_S_12	=	B'00101100'		; UPDATE SERVO 12
;_S_13	=	B'00101101'		; UPDATE SERVO 13
;_S_14	=	B'00101110'		; UPDATE SERVO 14
;_S_15	=	B'00101111'		; UPDATE SERVO 15

;***************************************************************
;
;	DECODIFICA ISTRUZIONI SERIALI
;
;***************************************************************

DECODE
        MOVFF	RCREG, TEMP     ;
;	MOVFF	TEMP, TXREG	; ECHO	<---------------

        BCF     STATUS, Z       ;
        MOVLW   _START		; Start Servo Update
        SUBWF   TEMP, W         ;
        BTFSC   STATUS, Z       ;
        GOTO	START		;

        BCF     STATUS, Z      ;
        MOVLW   _START_MOV	; Start MOV Update
        SUBWF   TEMP, W        ;
        BTFSC   STATUS, Z      ;
        GOTO	START_MOV	;
        
        BCF     STATUS, Z       ;
        MOVLW   _STOP		; Stop Servo Update
        SUBWF   TEMP, W         ;
        BTFSC   STATUS, Z       ;
        GOTO 	STOP		;

        BCF     STATUS, Z       ;
        MOVLW   _STOP_MOV	; Stop MOV Update
        SUBWF   TEMP, W         ;
        BTFSC   STATUS, Z       ;
        GOTO 	STOP_MOV	;

        BCF     STATUS, Z       ;
        MOVLW   _SPEED_MOV	; UPDATE via PC SPEED
        SUBWF   TEMP, W         ;
        BTFSC   STATUS, Z       ;
        GOTO	SPEED_MOV	;

        BCF     STATUS, Z       ;
        MOVLW   _S_0		; UPDATE via PC SERVO_0
        SUBWF   TEMP, W         ;
        BTFSC   STATUS, Z       ;
        GOTO	SERVO_0		;

        BCF     STATUS, Z       ;
        MOVLW   _S_1		; UPDATE via PC SERVO_1
        SUBWF   TEMP, W         ;
        BTFSC   STATUS, Z       ;
        GOTO	SERVO_1		;

        BCF     STATUS, Z       ;
        MOVLW   _S_2		; UPDATE via PC SERVO_2
        SUBWF   TEMP, W         ;
        BTFSC   STATUS, Z       ;
        GOTO	SERVO_2		;

        BCF     STATUS, Z       ;
        MOVLW   _S_3		; UPDATE via PC SERVO_3
        SUBWF   TEMP, W         ;
        BTFSC   STATUS, Z       ;
        GOTO	SERVO_3		;

        BCF     STATUS, Z       ;
        MOVLW   _S_4		; UPDATE via PC SERVO_4
        SUBWF   TEMP, W         ;
        BTFSC   STATUS, Z       ;
        GOTO	SERVO_4		;

        BCF     STATUS, Z       ;
        MOVLW   _S_5		; UPDATE via PC SERVO_5
        SUBWF   TEMP, W         ;
        BTFSC   STATUS, Z       ;
        GOTO	SERVO_5		;
				;
        BCF     STATUS, Z       ;
        MOVLW   _S_6		; UPDATE via PC SERVO_6
        SUBWF   TEMP, W         ;
        BTFSC   STATUS, Z       ;
        GOTO	SERVO_6		;

        BCF     STATUS, Z       ;
        MOVLW   _S_7		; UPDATE via PC SERVO_7
        SUBWF   TEMP, W         ;
        BTFSC   STATUS, Z       ;
        GOTO	SERVO_7		;
				;
        BCF     STATUS, Z       ;
        MOVLW   _S_8		; UPDATE via PC SERVO_8
        SUBWF   TEMP, W         ;
        BTFSC   STATUS, Z       ;
        GOTO	SERVO_8		;

        BCF     STATUS, Z       ;
        MOVLW   _S_9		; UPDATE via PC SERVO_9
        SUBWF   TEMP, W         ;
        BTFSC   STATUS, Z       ;
        GOTO	SERVO_9		;

        BCF     STATUS, Z       ;
        MOVLW   _S_10		; UPDATE via PC SERVO_10
        SUBWF   TEMP, W         ;
        BTFSC   STATUS, Z       ;
        GOTO	SERVO_10	;

        BCF     STATUS, Z       ;
        MOVLW   _S_11		; UPDATE via PC SERVO_11
        SUBWF   TEMP, W         ;
        BTFSC   STATUS, Z       ;
        GOTO	SERVO_11	;
				;
	BCF	STATUS, Z	; LOAD PROGRAM
	MOVLW	_LOAD		; 
	SUBWF	TEMP, W		;
	BTFSC	STATUS, Z	;
	GOTO	LOAD_PROG	; 
				;
	BCF	STATUS, Z	; DOWNLOAD PROGRAM
	MOVLW	_DOWNLOAD	; 
	SUBWF	TEMP, W		;
	BTFSC	STATUS, Z	;
	GOTO	SEND_PROG	; 
				;
	BCF	STATUS, Z	; ERASE PROGRAM
	MOVLW	_ERASE		; 
	SUBWF	TEMP, W		;
	BTFSC	STATUS, Z	;
	GOTO	ERASE		; 

	BCF	STATUS, Z	;
	MOVLW	_RESET		; RESET PROCESSOR
	SUBWF	TEMP, W		;
	BTFSC	STATUS, Z	;
	RESET			; 

;	BCF	STATUS, Z	;
;	MOVLW	_PC_EMU		;
;	SUBWF	TEMP, W		;
;	BTFSC	STATUS, Z	;
;	BRA	PRE_RC_CONTROL	; 
;
	RETURN			;

;PRE_RC_CONTROL
;	BTFSS	PIR1, RCIF	;
;	BRA	$-2		; WAIT NEXT BYTE
;RC_CONTROL
;        MOVFF	RCREG, TEMP     ;
;	BTFSS	PIR1, RCIF	;
;	BRA	$-2		; WAIT NEXT BYTE
;        MOVF	RCREG, W	;
;	COMF	RCREG, W	;
;	SUBWF	TEMP, W		; IF OK RESULTS = 0
;	BTFSC	STATUS, Z		;	
;	MOVFF	TEMP, STATUS_SEQ_NUM	;
;
;	BTFSS	MOV, 0		; FIRST TIME INITIALIZATION!!!
;	CALL	TIMER_MOV_ON	;
;
	RETURN			;

;*******************************;

;	BCF	STATUS, Z	;
;	MOVLW	.50		; ...
;	SUBWF	TEMP, W		;
;	BTFSC	STATUS, Z	;
;	CALL	....		; 


SPEED_MOV	
;	BTFSS	PIR1, RCIF		;
;	BRA	SPEED_MOV		; ATTENDO IL DUTY PER IL PWM
;	MOVFF	RCREG, SPEED		; METTO IL VALORE DELLA VELOCITA' NEL REGISTRO GIUSTO
	RETURN


SERVO_0	BTFSS	PIR1, RCIF		;
	BRA	SERVO_0			; ATTENDO IL DUTY PER IL PWM
	MOVFF	RCREG, VALUE_SERV_0	; METTO IL VALORE DEL DUTY NEL REGISTRO GIUSTO
	RETURN

SERVO_1	BTFSS	PIR1, RCIF		;
	BRA	SERVO_1			; ATTENDO IL DUTY PER IL PWM
	MOVFF	RCREG, VALUE_SERV_1	; METTO IL VALORE DEL DUTY NEL REGISTRO GIUSTO
	RETURN

SERVO_2	BTFSS	PIR1, RCIF		;
	BRA	SERVO_2			; ATTENDO IL DUTY PER IL PWM
	MOVFF	RCREG, VALUE_SERV_2	; METTO IL VALORE DEL DUTY NEL REGISTRO GIUSTO
	RETURN

SERVO_3	BTFSS	PIR1, RCIF		;
	BRA	SERVO_3			; ATTENDO IL DUTY PER IL PWM
	MOVFF	RCREG, VALUE_SERV_3	; METTO IL VALORE DEL DUTY NEL REGISTRO GIUSTO
	RETURN

SERVO_4	BTFSS	PIR1, RCIF		;
	BRA	SERVO_4			; ATTENDO IL DUTY PER IL PWM
	MOVFF	RCREG, VALUE_SERV_4	; METTO IL VALORE DEL DUTY NEL REGISTRO GIUSTO
	RETURN

SERVO_5	BTFSS	PIR1, RCIF		;
	BRA	SERVO_5			; ATTENDO IL DUTY PER IL PWM
	MOVFF	RCREG, VALUE_SERV_5	; METTO IL VALORE DEL DUTY NEL REGISTRO GIUSTO
	RETURN

SERVO_6	BTFSS	PIR1, RCIF		;
	BRA	SERVO_6			; ATTENDO IL DUTY PER IL PWM
	MOVFF	RCREG, VALUE_SERV_6	; METTO IL VALORE DEL DUTY NEL REGISTRO GIUSTO
	RETURN

SERVO_7	BTFSS	PIR1, RCIF		;
	BRA	SERVO_7			; ATTENDO IL DUTY PER IL PWM
	MOVFF	RCREG, VALUE_SERV_7	; METTO IL VALORE DEL DUTY NEL REGISTRO GIUSTO
	RETURN

SERVO_8	BTFSS	PIR1, RCIF		;
	BRA	SERVO_8			; ATTENDO IL DUTY PER IL PWM
	MOVFF	RCREG, VALUE_SERV_8	; METTO IL VALORE DEL DUTY NEL REGISTRO GIUSTO
	RETURN

SERVO_9	BTFSS	PIR1, RCIF		;
	BRA	SERVO_9			; ATTENDO IL DUTY PER IL PWM
	MOVFF	RCREG, VALUE_SERV_9	; METTO IL VALORE DEL DUTY NEL REGISTRO GIUSTO
	RETURN

SERVO_10 BTFSS	PIR1, RCIF		;
	BRA	SERVO_10		; ATTENDO IL DUTY PER IL PWM
	MOVFF	RCREG, VALUE_SERV_10	; METTO IL VALORE DEL DUTY NEL REGISTRO GIUSTO
	RETURN

SERVO_11 BTFSS	PIR1, RCIF		;
	BRA	SERVO_11		; ATTENDO IL DUTY PER IL PWM
	MOVFF	RCREG, VALUE_SERV_11	; METTO IL VALORE DEL DUTY NEL REGISTRO GIUSTO
	RETURN

START					;
	CALL	TIMER_ON		;
	RETURN				;

START_MOV
	BTFSS	PIR1, RCIF		; WAIT FOR STATUS_SEQ_NUM VALUE TO UPDATE
	BRA	$-2			;
	MOVF	RCREG, W		;
	MOVWF	EMU_STATUS_SEQ_NUM	;
	MOVWF	STATUS_SEQ_NUM		;
	SETF	EMU_MODE		;
	CALL	TIMER_MOV_ON		;
	RETURN				;

STOP				
	CALL	TIMER_OFF		;
	RETURN				;

STOP_MOV
	CLRF	STATUS_SEQ_NUM		;
	CLRF	EMU_MODE		;
	CALL	TIMER_MOV_OFF		;
	RETURN				;

;****************************************************************************
;****************************************************************************


LOAD_PROG:

	BSF	R_LED				;
	LFSR	0, TOC_RAM			; SETTING RAM POINTER
	MOVLW	(MAX_NUM_OF_SEQ*D'4')+D'2'	;
	MOVWF	TEMP_COUNTER_L			;
TOC_INLOOP					;
	BTFSS	PIR1, RCIF			;<; WAIT FOR TOC BYTES
	BRA	$-2				; ;
	MOVF	RCREG, W			; ;
	MOVWF	POSTINC0			; ; 
	DECFSZ	TEMP_COUNTER_L, F		; ;
	BRA	TOC_INLOOP			;>; RAM TOC READY
						;
	MOVLW	0xFF				; ACK
	MOVWF	TXREG				;

	MOVF	TOT_NUM_FBYTE_H, W		;
	MOVWF	TEMP_COUNTER_H 			; TOT_NUM_FBYTE_H ---> TEMP_COUNTER_H
	MOVF	TOT_NUM_FBYTE_L, W		;
	MOVWF	TEMP_COUNTER_L 			; TOT_NUM_FBYTE_L ---> TEMP_COUNTER_L
						;
	TabSetting (FLASH_OFFSET - D'8')	;  OFFESET ADJUST! (FOR BUFFER LOADING SHIFT)
PAGE_LOOP					;<----------------------------------------------|
	LFSR	0, FLASH_BUFF			; SETTING RAM POINTER				|
	MOVF	TEMP_COUNTER_L, W		;						|
	MOVWF	PAGE_COUNTER 			;						|
	MOVF	TEMP_COUNTER_H, F		; IF TOT_NUM_FBYTE_H <> 0 ...			|
	BZ	RX_PAGE_LOOP			; ... THEN PAGE_COUNTER == 256			|
	CLRF	PAGE_COUNTER			; ELSE PAGE_COUNTER == TOT_NUM_FBYTE_L		|
RX_PAGE_LOOP					;						|
	BTFSS	PIR1, RCIF			;<						|
	BRA	$-2				;; ATTENDO IL NUMERO DI BYTE			|
	MOVF	RCREG, W			;;						|
	MOVWF	POSTINC0			;; IN RAM WHIT POST INCREMENT			|
	DECFSZ	PAGE_COUNTER			;; 256 CICLI!					|
	BRA	RX_PAGE_LOOP			;> PAGE_COUNTER					|
						;						|
	LFSR	0, FLASH_BUFF			; SETTING RAM POINTER				|
	CLRF	BYTE_NUM			; SETTING NUM OF BYTE TO WRITE (0 = 256)	|
	CALL	WRITE_FLASH_NO_ADDR_NO_OFFSET	;						|
						;						|
	MOVLW	0xFF				; ACK						|
	MOVWF	TXREG				;						|
						;						|
	MOVF	TEMP_COUNTER_H, F		;						|
	BZ	EXIT_LOOP			;						|
	DECF	TEMP_COUNTER_H, F		;						|
	BRA	PAGE_LOOP			;>----------------------------------------------|
EXIT_LOOP					;
	CALL	SAVE_TOC			; TOC RAM --> FLASH
	CALL	RES_VAL				;
	BCF	R_LED				;
	RETURN					;

;---;---;---;---;---;---;---;---;---;---;---
;--- Controllare per piccoli valori!!!!!;---
;---;---;---;---;---;---;---;---;---;---;---
SEND_PROG:
	BSF	R_LED				;
	TabSetting FLASH_INDEX_OFFSET		;
						;
	MOVLW	(MAX_NUM_OF_SEQ*D'4')+D'2'	; 66 BYTES SENT
	MOVWF	TEMP_COUNTER_L			;
TOC_OUTLOOP					;
	TBLRD*+					;<;
	MOVF	TABLAT, W			; ;
	CALL	RS232PUTCHAR			; ;
	DECFSZ	TEMP_COUNTER_L, F		; ;
	BRA	TOC_OUTLOOP			;>; FLASH TOC SENT
						;
	DelayMilliSec	D'500'			;
						;
	TabSetting	FLASH_OFFSET		; SAND FLASH BYTES
	MOVF	TOT_NUM_FBYTE_H, W		;
	MOVWF	TEMP_COUNTER_H 			;
	MOVF	TOT_NUM_FBYTE_L, W		;
	MOVWF	TEMP_COUNTER_L 			;
SEND_LOOP					;<;
	TBLRD*+					; ;
	MOVF	TABLAT, W			; ;
	CALL	RS232PUTCHAR			; ;
	DECF	TEMP_COUNTER_L, F		; ; DECREMENT 2 BYTE NUMBER <---- DEC BORRW?
	MOVLW	0x00				; ;
	SUBWFB	TEMP_COUNTER_H, F		; ;
	MOVF	TEMP_COUNTER_H, W		; ; TEST IF ZERO
	IORWF	TEMP_COUNTER_L, W		; ;
	BZ	EXIT_SEND_LOOP			; ;
	BRA	SEND_LOOP			;>;
EXIT_SEND_LOOP					;
	TabSetting	FLASH_OFFSET		;
	CALL	RES_VAL				;
	BCF	R_LED				;
	RETURN					;




ERASE:
	BSF	R_LED						;
	TabSetting FLASH_INDEX_OFFSET				;
	MOVLW	((((NUM_OF_SERVO_CHANN + 2)*256*16)/64)+3)	;
	MOVWF	BYTE_NUM					;
	BSF	R_LED						;
	CALL	ERASE_FLASH_NO_ADDR				;
	MOVLW	0xFF						; ACK
	MOVWF	TXREG						;
	DelayMilliSec	D'100'					;
	BCF	R_LED						;
	RETURN							;



;****************************************************************************
;****************************************************************************

RS232PUTCHAR:
	BTFSS	PIR1, TXIF		;
	BRA	$ - 2			;
	MOVWF	TXREG			;
	RETURN				;


;****************************************************************************
;****************************************************************************

DELAY_1_SECONDO
	DelayMilliSec	D'1000'	;; 1 secondo di attesa
	RETURN

;****************************************************************************
;****************************************************************************

F_SERV	MACRO					;
	LOCAL	a = 0				;
	WHILE	a < NUM_OF_SERVO_CHANN		;
	TBLRD*+					;
	MOVF	TABLAT, W			;
	MOVWF	VALUE_SERV_#v(a)		;
	a ++					;
	ENDW					;
	ENDM


READ_STEP_FROM_FLASH
	F_SERV
	TBLRD*+					;
	MOVF	TABLAT, W			;
	MOVWF	TIME_STEP_H			;
	TBLRD*+					;
	MOVF	TABLAT, W			;
	MOVWF	TIME_STEP_L			;
	RETURN
	
	
